@@ -26,9 +26,7 @@
 #include <glib/gstdio.h>
 #include <gnome.h>
 #include <pwd.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <gio/gio.h>
 #include <libgnomeui/gnome-thumbnail.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
@@ -44,6 +42,8 @@
 #define MAX_HEIGHT 150
 #define MAX_WIDTH  150
 
+#define EMAIL_SLOTS 4
+
 typedef struct {
 	EContact 	*contact;
 	EBook    	*book;
@@ -56,6 +56,8 @@ typedef struct {
 
 	EContactAddress *addr1;
 	EContactAddress *addr2;
+	gchar           *email[EMAIL_SLOTS];
+	const gchar     *email_types[EMAIL_SLOTS];
 
 	gboolean      	 have_image;
 	gboolean      	 image_changed;
@@ -84,15 +86,15 @@ enum {
 	ADDRESS_COUNTRY
 };
 
-#define EMAIL_HOME              0
-#define EMAIL_WORK              1
+#define EMAIL_WORK              0
+#define EMAIL_HOME              1
 #define ADDRESS_HOME		21
 #define ADDRESS_WORK		27
 
 struct WidToCid ids[] = {
 
-	{ "email-work-e",      E_CONTACT_EMAIL_1             },	/* 00 */
-	{ "email-home-e",      E_CONTACT_EMAIL_2             }, /* 01 */
+	{ "email-work-e",      0                             }, /* 00 */
+	{ "email-home-e",      1                             }, /* 01 */
 
 	{ "phone-home-e",      E_CONTACT_PHONE_HOME          }, /* 02 */
 	{ "phone-mobile-e",    E_CONTACT_PHONE_MOBILE        }, /* 03 */
@@ -136,6 +138,7 @@ struct WidToCid ids[] = {
 
 #define ATTRIBUTE_HOME "HOME"
 #define ATTRIBUTE_WORK "WORK"
+#define ATTRIBUTE_OTHER "OTHER"
 
 static void about_me_set_address_field (EContactAddress *, guint, gchar *);
 
@@ -169,23 +172,54 @@ about_me_destroy (GnomeAboutMe *me)
 	if (me->dialog)
 		g_object_unref (me->dialog);
 
+	g_free (me->email[0]);
+	g_free (me->email[1]);
+	g_free (me->email[2]);
+	g_free (me->email[3]);
+
 	g_free (me->person);
 	g_free (me->login);
 	g_free (me->username);
 	g_free (me);
 }
 
+static void
+about_me_update_email (GnomeAboutMe *me)
+{
+	GList *attrs = NULL;
+	gint i;
+
+	for (i = 0; i < EMAIL_SLOTS; ++i) {
+		if (me->email[i] != NULL) {
+			EVCardAttribute *attr;
+			const gchar *type = me->email_types[i];
+
+			attr = e_vcard_attribute_new (NULL, EVC_EMAIL);
+
+			e_vcard_attribute_add_param_with_value (attr,
+								e_vcard_attribute_param_new (EVC_TYPE),
+								type ? type : ATTRIBUTE_OTHER);
+
+			e_vcard_attribute_add_value (attr, me->email[i]);
+			attrs = g_list_append (attrs, attr);
+		}
+	}
+
+	e_contact_set_attributes (me->contact, E_CONTACT_EMAIL, attrs);
+
+	g_list_foreach (attrs, (GFunc) e_vcard_attribute_free, NULL);
+	g_list_free (attrs);
+}
+
 static void
 about_me_commit (GnomeAboutMe *me)
 {
 	EContactName *name;
-	GError *error;
 
 	char *strings[4], **stringptr;
 	char *fileas;
 
 	name = NULL;
-	error = NULL;
 
 	if (me->create_self) {
 		if (me->username == NULL)
@@ -210,12 +244,14 @@ about_me_commit (GnomeAboutMe *me)
 		g_free (fileas);
 	}
 
+	about_me_update_email (me);
+
 	if (me->create_self) {
-		e_book_add_contact (me->book, me->contact, &error);
-		e_book_set_self (me->book, me->contact, &error);
+		e_book_add_contact (me->book, me->contact, NULL);
+		e_book_set_self (me->book, me->contact, NULL);
 	} else {
-		if (e_book_commit_contact (me->book, me->contact, &error) == FALSE)
-			g_print ("There was an undetermined error\n");
+		if (!e_book_commit_contact (me->book, me->contact, NULL))
+			g_warning ("Could not save contact information");
 	}
 
 	me->create_self = FALSE;
@@ -232,7 +268,7 @@ about_me_commit_from_timeout (GnomeAboutMe *me)
 static gboolean
 about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 {
-	gchar *str = NULL;
+	gchar *str;
 	const gchar *wid;
 	gint i;
 
@@ -242,13 +278,9 @@ about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 		return FALSE;
 
 	for (i = 0; ids[i].wid != NULL; i++)
-		if (g_ascii_strcasecmp (ids[i].wid, wid) == 0)
+		if (strcmp (ids[i].wid, wid) == 0)
 			break;
 
-	if (ids[i].cid == 0) {
-		return FALSE;
-	}
-
 	if (GTK_IS_ENTRY (widget)) {
 		str = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
 	} else if (GTK_IS_TEXT_VIEW (widget)) {
@@ -262,11 +294,18 @@ about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 		gtk_text_iter_forward_to_end (&iter_end);
 		str = gtk_text_iter_get_text (&iter_start, &iter_end);
 	} else {
-		str = g_strdup ("");
+		return FALSE;
 	}
 
+	if (i == EMAIL_HOME || i == EMAIL_WORK) {
+
+		g_free (me->email[ids[i].cid]);
+		if (str[0] == '\0')
+			me->email[ids[i].cid] = NULL;
+		else
+			me->email[ids[i].cid] = g_strdup (str);
 	/* FIXME: i'm getting an empty address field in evolution */
-	if (i >= ADDRESS_HOME && i < ADDRESS_WORK) {
+	} else if (i >= ADDRESS_HOME && i < ADDRESS_WORK) {
 		about_me_set_address_field (me->addr1, ids[i].cid, str);
 		e_contact_set (me->contact, E_CONTACT_ADDRESS_HOME, me->addr1);
 	} else if (i >= ADDRESS_WORK) {
@@ -294,7 +333,7 @@ get_user_login (void)
 	struct passwd pwd, *err;
 
 	int i;
-#if __sun
+#if defined(__sun) && !defined(_POSIX_PTHREAD_SEMANTICS)
 	i = getpwuid_r (getuid (), &pwd, buf, sizeof (buf));
 	return (i != 0) ? g_strdup (pwd.pw_name) : NULL;
 #else
@@ -348,64 +387,77 @@ about_me_set_address_field (EContactAddress *addr, guint cid, gchar *str)
 {
 	switch (cid) {
 		case ADDRESS_STREET:
-			if (addr->street)
-				g_free (addr->street);
+			g_free (addr->street);
 			addr->street = g_strdup (str);
 			break;
 		case ADDRESS_POBOX:
-			if (addr->po)
-				g_free (addr->po);
+			g_free (addr->po);
 			addr->po = g_strdup (str);
 			break;
 		case ADDRESS_LOCALITY:
-			if (addr->locality)
-				g_free (addr->locality);
+			g_free (addr->locality);
 			addr->locality = g_strdup (str);
 			break;
 		case ADDRESS_CODE:
-			if (addr->code)
-				g_free (addr->code);
+			g_free (addr->code);
 			addr->code = g_strdup (str);
 			break;
 		case ADDRESS_REGION:
-			if (addr->region)
-				g_free (addr->region);
+			g_free (addr->region);
 			addr->region = g_strdup (str);
 			break;
 		case ADDRESS_COUNTRY:
-			if (addr->country)
-				g_free (addr->country);
+			g_free (addr->country);
 			addr->country = g_strdup (str);
 			break;
 	}
 }
 
 static void
-about_me_setup_email (EContact *contact)
+about_me_setup_email (GnomeAboutMe *me)
 {
-	GList *attrs, *a;
+	GList *attrs, *la;
 	gboolean has_home = FALSE, has_work = FALSE;
-	guint cid;
+	guint i;
 
-	attrs = e_contact_get_attributes (contact, E_CONTACT_EMAIL);
+	attrs = e_contact_get_attributes (me->contact, E_CONTACT_EMAIL);
 
-	for (a = attrs, cid = E_CONTACT_EMAIL_1; a; a = a->next, ++cid) {
-		EVCardAttribute *att = (EVCardAttribute *) a->data;
+	for (la = attrs, i = 0; la; la = la->next, ++i) {
+		EVCardAttribute *a = la->data;
 
-		if (!has_home &&
-		    e_vcard_attribute_has_type (att, ATTRIBUTE_HOME)) {
-			has_home = TRUE;
-			ids[EMAIL_HOME].cid = cid;
-		} else if (!has_work &&
-			   e_vcard_attribute_has_type (att, ATTRIBUTE_WORK)) {
-			has_work = TRUE;
-			ids[EMAIL_WORK].cid = cid;
+		me->email[i] = e_vcard_attribute_get_value (a);
+		if (e_vcard_attribute_has_type (a, ATTRIBUTE_HOME)) {
+			me->email_types[i] = ATTRIBUTE_HOME;
+			if (!has_home) {
+				ids[EMAIL_HOME].cid = i;
+				has_home = TRUE;
+			}
+		} else if (e_vcard_attribute_has_type (a, ATTRIBUTE_WORK)) {
+			me->email_types[i] = ATTRIBUTE_WORK;
+			if (!has_work) {
+				ids[EMAIL_WORK].cid = i;
+				has_work = TRUE;
+			}
+		} else {
+			me->email_types[i] = ATTRIBUTE_OTHER;
 		}
-		e_vcard_attribute_free (att);
+
+		e_vcard_attribute_free (a);
 	}
 
 	g_list_free (attrs);
+
+	if (ids[EMAIL_HOME].cid == ids[EMAIL_WORK].cid) {
+		if (has_home)
+			ids[EMAIL_WORK].cid = 1;
+		else
+			ids[EMAIL_HOME].cid = 0;
+	}
+
+	me->email_types[ids[EMAIL_WORK].cid] = ATTRIBUTE_WORK;
+	me->email_types[ids[EMAIL_HOME].cid] = ATTRIBUTE_HOME;
 }
+
 /**
  * about_me_load_string_field:
  *
@@ -430,7 +482,9 @@ about_me_load_string_field (GnomeAboutMe *me, const gchar *wid, guint cid, guint
 		return;
 	}
 
-	if (aid >= ADDRESS_HOME && aid < ADDRESS_WORK) {
+	if (aid == EMAIL_HOME || aid == EMAIL_WORK) {
+		str = e_contact_get_const (me->contact, E_CONTACT_EMAIL_1 + cid);
+	} else if (aid >= ADDRESS_HOME && aid < ADDRESS_WORK) {
 		str = about_me_get_address_field (me->addr1, cid);
 	} else if (aid >= ADDRESS_WORK) {
 		str = about_me_get_address_field (me->addr2, cid);
@@ -463,7 +517,8 @@ about_me_load_photo (GnomeAboutMe *me, EContact *contact)
 
 	widget = WID ("image-chooser");
 
-	e_image_chooser_set_from_file (E_IMAGE_CHOOSER (widget), me->person);
+	if (me->person)
+		e_image_chooser_set_from_file (E_IMAGE_CHOOSER (widget), me->person);
 
 	photo = e_contact_get (contact, E_CONTACT_PHOTO);
 
@@ -620,20 +675,40 @@ about_me_update_preview (GtkFileChooser *chooser,
 
 	if (uri) {
 		GtkWidget *image;
-		GdkPixbuf *pixbuf;
-		gchar *mime_type;
+		GdkPixbuf *pixbuf = NULL;
+		GFile *file;
+		GFileInfo *file_info;
 
 		if (!me->thumbs)
 			me->thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
 
+		file = g_file_new_for_uri (uri);
+		file_info = g_file_query_info (file,
+					       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					       G_FILE_QUERY_INFO_NONE,
+					       NULL, NULL);
+		g_object_unref (file);
+
+		if (file_info != NULL) {
+			const gchar *content_type;
+
+			content_type = g_file_info_get_content_type (file_info);
+			if (content_type) {
+				gchar *mime_type;
+
+				mime_type = g_content_type_get_mime_type (content_type);
+
+				pixbuf = gnome_thumbnail_factory_generate_thumbnail (me->thumbs,
+										     uri,
+										     mime_type);
+				g_free (mime_type);
+			}
+			g_object_unref (file_info);
+		}
 
-		mime_type = gnome_vfs_get_mime_type (uri);
-		pixbuf = gnome_thumbnail_factory_generate_thumbnail (me->thumbs,
-								     uri,
-								     mime_type);
 		image = gtk_file_chooser_get_preview_widget (chooser);
 
-		if(pixbuf != NULL) {
+		if (pixbuf != NULL) {
 			gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
 			g_object_unref (pixbuf);
 		} else {
@@ -641,7 +716,6 @@ about_me_update_preview (GtkFileChooser *chooser,
 						  "gtk-dialog-question",
 						  GTK_ICON_SIZE_DIALOG);
 		}
-		g_free (mime_type);
 	}
 	gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
 }
@@ -790,6 +864,7 @@ about_me_setup_dialog (void)
 	GtkIconInfo  *icon;
 	GladeXML     *dialog;
 	GError 	     *error = NULL;
+	GList        *chain;
 
 	struct passwd *pwent;
 	gchar *str;
@@ -809,7 +884,7 @@ about_me_setup_dialog (void)
 
 	/* Connect the close button signal */
 	main_dialog = WID ("about-me-dialog");
-	g_signal_connect (G_OBJECT (main_dialog), "response",
+	g_signal_connect (main_dialog, "response",
 			  G_CALLBACK (about_me_button_clicked_cb), me);
 
 	gtk_window_set_resizable (GTK_WINDOW (main_dialog), FALSE);
@@ -820,10 +895,10 @@ about_me_setup_dialog (void)
 	me->theme = gtk_icon_theme_get_for_screen (me->screen);
 
 	icon = gtk_icon_theme_lookup_icon (me->theme, "stock_person", 80, 0);
-
-	me->person = g_strdup (gtk_icon_info_get_filename (icon));
-
-	gtk_icon_info_free (icon);
+	if (icon != NULL) {
+		me->person = g_strdup (gtk_icon_info_get_filename (icon));
+		gtk_icon_info_free (icon);
+	}
 
 	g_signal_connect_object (me->theme, "changed",
 				 G_CALLBACK (about_me_icon_theme_changed),
@@ -859,7 +934,7 @@ about_me_setup_dialog (void)
 			}
 		}
 	} else {
-		about_me_setup_email (me->contact);
+		about_me_setup_email (me);
 	}
 
 	/************************************************/
@@ -876,7 +951,7 @@ about_me_setup_dialog (void)
 
 	/************************************************/
 
-	if (tok[0] == NULL || strlen (tok[0]) == 0)
+	if (tok[0] == NULL || *tok[0] == '\0')
 		me->username = NULL;
 	else
 		me->username = g_strdup (tok[0]);
@@ -885,7 +960,7 @@ about_me_setup_dialog (void)
 	about_me_load_photo (me, me->contact);
 
 	widget = WID ("fullname");
-	if (tok[0] == NULL || strlen (tok[0]) == 0) {
+	if (tok[0] == NULL || *tok[0] == '\0') {
 		str = g_strdup_printf ("<b><span size=\"xx-large\">%s</span></b>", me->login);
 	} else {
 		str = g_strdup_printf ("<b><span size=\"xx-large\">%s</span></b>", tok[0]);
@@ -897,7 +972,7 @@ about_me_setup_dialog (void)
 	widget = WID ("login");
 	gtk_label_set_text (GTK_LABEL (widget), me->login);
 
-	if (tok[0] == NULL || strlen (tok[0]) == 0) {
+	if (tok[0] == NULL || *tok[0] == '\0') {
 		str = g_strdup_printf (_("About %s"), me->login);
 	} else {
 		str = g_strdup_printf (_("About %s"), tok[0]);
@@ -907,17 +982,38 @@ about_me_setup_dialog (void)
 	g_strfreev (tok);
 
 	widget = WID ("password");
-	g_signal_connect (G_OBJECT (widget), "clicked",
+	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (about_me_passwd_clicked_cb), me);
 
 	widget = WID ("button-image");
-	g_signal_connect (G_OBJECT (widget), "clicked",
+	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (about_me_image_clicked_cb), me);
 
 	widget = WID ("image-chooser");
-	g_signal_connect (G_OBJECT (widget), "changed",
+	g_signal_connect (widget, "changed",
 			  G_CALLBACK (about_me_image_changed_cb), me);
 
+	/* Address tab: set up the focus chains */
+	chain = g_list_prepend (NULL, WID ("addr-country-1"));
+	chain = g_list_prepend (chain, WID ("addr-po-1"));
+	chain = g_list_prepend (chain, WID ("addr-region-1"));
+	chain = g_list_prepend (chain, WID ("addr-code-1"));
+	chain = g_list_prepend (chain, WID ("addr-locality-1"));
+	chain = g_list_prepend (chain, WID ("addr-scrolledwindow-1"));
+	widget = WID ("addr-table-1");
+	gtk_container_set_focus_chain (GTK_CONTAINER (widget), chain);
+	g_list_free (chain);
+
+	chain = g_list_prepend (NULL, WID ("addr-country-2"));
+	chain = g_list_prepend (chain, WID ("addr-po-2"));
+	chain = g_list_prepend (chain, WID ("addr-region-2"));
+	chain = g_list_prepend (chain, WID ("addr-code-2"));
+	chain = g_list_prepend (chain, WID ("addr-locality-2"));
+	chain = g_list_prepend (chain, WID ("addr-scrolledwindow-2"));
+	widget = WID ("addr-table-2");
+	gtk_container_set_focus_chain (GTK_CONTAINER (widget), chain);
+	g_list_free (chain);
+
 	about_me_load_info (me);
 
 	gtk_widget_show_all (main_dialog);
